-- FooBarModule.lua
-- This file contains the FooBarModule class.
-- The FooBarModule class is a module that docks to the default FooBar.
-- They can have events handed to them and consist of three frames - 
-- 	* DockingFrame (handles events, everything is "docked" to this)
--	* TextureFrame (handles textures, placed above DockingFrame).
--	* TextFrame (handles text, placed above TextureFrame).

FooBarModule = {
	mNumModules = 0,
	functions = {}, -- this is pointed to by __index in FooBarModules
	modules = {};
};

function FooBarModule:containsModule(name)
	for k, v in pairs(FooBarModule.modules) do
		if v._moduleName == name then
			return v;
		end
	end
	return false;
end

-- Creates and hooks a new FooBarModule.
function FooBarModule:new(name, parent)
	if parent == nil then
		parent = FooBar.getFoobar();
	end
	
	if FooBarModule:containsModule(name) then
		return FooBarModule:containsModule(name);
	end
	
	local x_padding, y_padding = FooBar:getXPadding(), FooBar:getYPadding();
	
	if FooBarModule.mNumModules > 0 then
		for i = 1, FooBarModule.mNumModules do
			x_padding = x_padding + FooBarModule.modules[i]:getWidth();
		end
	end
	
	FooBarModule.mNumModules = FooBarModule.mNumModules + 1;
	
	local o = {
		frames = {},
		_type = "FooBarModule",
		_index = FooBarModule.mNumModules,
		_prefix = "",
		_moduleName = name,
	};
	
	-- initialize these to allow access to each other.
	o.frames.mDockingFrame = UI.CreateFrame("Frame", "FooBarModule "..FooBarModule.mNumModules, parent);
	o.frames.mDockingFrame:SetHeight(parent:GetHeight());
	
	o.frames.mDockingFrame:SetPoint("TOPLEFT", parent, "TOPLEFT", x_padding, y_padding);
		
	o.frames.mTextureFrame = UI.CreateFrame("Texture", "FooBarModule "..FooBarModule.mNumModules.." texture frame", o.frames.mDockingFrame);
	o.frames.mTextureFrame:SetPoint("TOPLEFT", o.frames.mDockingFrame, "TOPLEFT");
	o.frames.mTextFrame = UI.CreateFrame("Text", "FooBarModule "..FooBarModule.mNumModules.." text frame", o.frames.mDockingFrame);
	o.frames.mTextFrame:SetPoint("TOPLEFT", o.frames.mDockingFrame, "TOPLEFT");
	o.frames.mTextFrame:SetFontSize(FooBar.FooBarGlobalTextSize);
	if FooBarModule.mNumModules > 1 then
		o._prefix = "| ";
	end
	o.frames.mTextFrame:SetText(o._prefix);
	
	-- Set layers.
	o.frames.mDockingFrame:SetLayer(FooBar.FooBarModuleLayer);	-- set this one to the default level.
	o.frames.mTextureFrame:SetLayer(FooBar.FooBarModuleLayer+1); -- display this above the docking frame.
	o.frames.mTextFrame:SetLayer(FooBar.FooBarModuleLayer+2); -- display this above the texture frame.
	
	
	setmetatable(o, {__index = FooBarModule.functions}); -- ready to deploy now.
	table.insert(FooBarModule.modules,  o);

	return FooBarModule.modules[#FooBarModule.modules]; -- return last element (should be this)
end

function FooBarModule.functions:getType()
	return self._type; 
end

-- Wrapper method to set the text of this frame.
-- This will also automatically resize the module if needed.
function FooBarModule.functions:setText(s)
	local otext = self.frames.mTextFrame:GetText();
	self.frames.mTextFrame:SetText(self._prefix..s);
	
	if FooBar.allowHeightResizeFromTextChange then
		self:getParent():SetHeight(self.frames.mTextFrame:GetFullHeight()+FooBar:getYPadding());
		FooBar:OnDimensionsChanged();
	end	
	
	self.frames.mDockingFrame:SetWidth( self.frames.mTextFrame:GetFullWidth() );
	self.frames.mTextureFrame:SetWidth( self.frames.mTextFrame:GetFullWidth() );
end

-- Wrapper method to get the width of this module.
-- This actually returns the width of the text frame.
function FooBarModule.functions:getWidth()
	return self.frames.mTextFrame:GetFullWidth();
end

-- Wrapper method to get the parent of this module.
function FooBarModule.functions:getParent()
	return self.frames.mDockingFrame:GetParent();
end

-- Wrapper method to set the text size.
function FooBarModule.functions:setTextSize(i)
	if FooBar.allowHeightResizeFromTextChange then
		self:getParent():SetHeight(self.frames.mTextFrame:GetFullHeight()+FooBar:getYPadding());
		FooBar:OnDimensionsChanged();
	end	
	self.frames.mTextFrame:SetFontSize(i);
end

function FooBarModule.functions:getModuleName()
	return self._moduleName;
end

function FooBarModule.functions:getTextFrame()
	return self.frames.mTextFrame;
end
function FooBarModule.functions.getDockingFrame()
	return self.frames.mDockingFrame;
end
function FooBarModule.functions:getTextureFrame()
	return self.frames.mTextureFrame;
end

-- Registers an Event for the Dock only. This can only change the Dock frame.
function FooBarModule.functions:registerEvent(eventName, func)
	if type(eventName) == "string" and type(func) == "function" then
		self.frames.mDockingFrame.Event[eventName] = func;
		return;
	end
	-- raise an error
end


-- Wrapper method for setting points
function FooBarModule.functions:setPoint(point1, anchor, point2, xoffset, yoffset)
	self.frames.mDockingFrame:SetPoint(point1,anchor,point2,xoffset,yoffset);
	self.frames.mTextureFrame:SetPoint(point1,anchor,point2,xoffset,yoffset);
	self.frames.mTextFrame:SetPoint(point1,anchor,point2,xoffset,yoffset);
end